# automatically generated by the FlatBuffers compiler, do not modify

# namespace: AOC

import flatbuffers

class MarketCoefficients(object):
    __slots__ = ['_tab']

    # MarketCoefficients
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # MarketCoefficients
    def Wood(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # MarketCoefficients
    def Food(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(4))
    # MarketCoefficients
    def Stone(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(8))

def CreateMarketCoefficients(builder, wood, food, stone):
    builder.Prep(4, 12)
    builder.PrependFloat32(stone)
    builder.PrependFloat32(food)
    builder.PrependFloat32(wood)
    return builder.Offset()
