# automatically generated by the FlatBuffers compiler, do not modify

# namespace: AOC

class TechState(object):
    CantResearch = 0
    CanResearchLater = 1
    CanResearch = 2
    Researching = 3
    Researched = 4

