# automatically generated by the FlatBuffers compiler, do not modify

# namespace: AOC

import flatbuffers

class VictoryPointsAndAttributes(object):
    __slots__ = ['_tab']

    # VictoryPointsAndAttributes
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # VictoryPointsAndAttributes
    def VpRelicsCaptured(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # VictoryPointsAndAttributes
    def VpRelicGold(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(4))
    # VictoryPointsAndAttributes
    def VpTradeProfit(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(8))
    # VictoryPointsAndAttributes
    def VpTributeReceived(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(12))
    # VictoryPointsAndAttributes
    def VpTributeSent(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(16))
    # VictoryPointsAndAttributes
    def VpTotalFood(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(20))
    # VictoryPointsAndAttributes
    def VpTotalWood(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(24))
    # VictoryPointsAndAttributes
    def VpTotalGold(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(28))
    # VictoryPointsAndAttributes
    def VpTotalStone(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(32))
    # VictoryPointsAndAttributes
    def AttrValueSpentObjects(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(36))
    # VictoryPointsAndAttributes
    def AttrValueSpentResearch(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(40))
    # VictoryPointsAndAttributes
    def AttrValueLostUnits(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(44))
    # VictoryPointsAndAttributes
    def AttrValueLostBuildings(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(48))
    # VictoryPointsAndAttributes
    def AttrValueCurrentUnits(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(52))
    # VictoryPointsAndAttributes
    def AttrValueCurrentBuildings(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(56))
    # VictoryPointsAndAttributes
    def AttrValueObjectsDestroyed(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(60))
    # VictoryPointsAndAttributes
    def AttrKills(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(64))
    # VictoryPointsAndAttributes
    def AttrDeaths(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(68))
    # VictoryPointsAndAttributes
    def AttrRazes(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(72))

def CreateVictoryPointsAndAttributes(builder, vpRelicsCaptured, vpRelicGold, vpTradeProfit, vpTributeReceived, vpTributeSent, vpTotalFood, vpTotalWood, vpTotalGold, vpTotalStone, attrValueSpentObjects, attrValueSpentResearch, attrValueLostUnits, attrValueLostBuildings, attrValueCurrentUnits, attrValueCurrentBuildings, attrValueObjectsDestroyed, attrKills, attrDeaths, attrRazes):
    builder.Prep(4, 76)
    builder.PrependFloat32(attrRazes)
    builder.PrependFloat32(attrDeaths)
    builder.PrependFloat32(attrKills)
    builder.PrependFloat32(attrValueObjectsDestroyed)
    builder.PrependFloat32(attrValueCurrentBuildings)
    builder.PrependFloat32(attrValueCurrentUnits)
    builder.PrependFloat32(attrValueLostBuildings)
    builder.PrependFloat32(attrValueLostUnits)
    builder.PrependFloat32(attrValueSpentResearch)
    builder.PrependFloat32(attrValueSpentObjects)
    builder.PrependFloat32(vpTotalStone)
    builder.PrependFloat32(vpTotalGold)
    builder.PrependFloat32(vpTotalWood)
    builder.PrependFloat32(vpTotalFood)
    builder.PrependFloat32(vpTributeSent)
    builder.PrependFloat32(vpTributeReceived)
    builder.PrependFloat32(vpTradeProfit)
    builder.PrependFloat32(vpRelicGold)
    builder.PrependFloat32(vpRelicsCaptured)
    return builder.Offset()
